#----------------------------------- - - - - -
#   .--.  __  __  __ _  ___  /..-   __                            
#  /___/ /_  /__+/_ /_\  /  /\   / /   /.-                     
# /   \ /__ /   /__/  / /  /  \ / /__ / \    
#
# "Eggdrops Most Effective Repeat/Flood handling"
#                    
# by Darren Coleman 1999
# durzel@barrysworld.com
# http://www.durzel.dircon.co.uk
#---------------------------------------------

#---------------------------------------------------------------------------------------------
# GLOBALS
#---------------------------------------------------------------------------------------------
# Version number
set version_num "v2.29"

# Repeatkick init timer (ignore)
set initstart 0

# Debug mode.
# 0 = No, 1 = Yes (default: NO)
set debug_mode 0

# Link/Sync with other bots running this script?
# 0 = No, 1 = Yes (default: YES)
set RKlinking 1

#---------------------------------------------------------------------------------------------
# DEFAULT CHANNEL SETTINGS (overwritten by .INI if one exists)
#---------------------------------------------------------------------------------------------
foreach chan [channels] {
   # --- Number of repeats before kicking
   set repeatkick($chan) 3
   # --- ..in how many seconds
   set repeattime($chan) 20
   # --- Repeatkick message
   set repeatmsg($chan) "Repeating"
   # --- Floodkick message
   set floodmsg($chan) "Flooding"
   # --- Ban on any repeat 
   set repeatban($chan) 0
   # --- Ban duration
   set repeatbantime($chan) 15
   # --- Repeatkick enabled/disabled (default: on)
   set repeatonoff($chan) 1
   # --- Floodkick enabled/disabled (default: on)
   set floodonoff($chan) 1
   # --- 'Character presence' tolerance
   set chartolerance($chan) 90
   # --- 'Positional matching' tolerance
   set postolerance($chan) 45
   # --- Logging of kicks (default: on)
   set logging($chan) 1
   # --- Timer ID
   set repeattimer($chan) 0
}

#---------------------------------------------------------------------------------------------
# Concurrency handling variables
#---------------------------------------------------------------------------------------------
# 'inuse' is used to 'lock' the repeat_num and repeat_last arrays from being reset (by the timer function)
# during the course of processing using values in these arrays (suppresses occasional TCL errors re: 'no
# such element in array')
set inuse 0

# 'timercalled' is used to denote that the timer *was* called during processing of a given function,
# so that the function itself knows it has to do whatever the timer was going to do after it has finished
# its own processing.
set timercalled 0

# Variable used to denote if another bot is processing a repeatkick/flood (ie. so that we do not process
# the command ourselves and punish the person twice).  Used only in bot-linked repeatkicking, ignore otherwise.
set bot_handling 0

# These two variables are used to control the way INI files are transferred between bots.  When each bot
# running this script starts up, it records a UNIX long integer time corresponding to when it 'finished'.
# Depending on the value of this, a bot will be either treated as a SLAVE (accepts data from the other), or
# MASTER (send the data).  'inichanged' is just used to show that a change was made to the INI file.
set inittime 0
set inichanged 0

#---------------------------------------------------------------------------------------------
# **IMPORTANT** INI/SCRIPT Locations
#---------------------------------------------------------------------------------------------
# It is IMPERATIVE that you set 'scriptdir' in your eggdrop.conf file to the location of the script
# directory.  The reason for doing this is two fold.  Firstly there can be (in my experience) a
# discrepancy between Windows and UNIX eggdrops with regard to path naming conventions, causing in
# some instances commands not to function correctly.

# The reason we set it in the eggdrop.conf, and not in this file is because this file can be
# overwritten by another bot with a newer version, which would write its Windows/UNIX-specific
# path with the file.  Make sense?.. :)

# Note: use 'set scriptdir "<full path>" in bot config
# example: /home/users/john/eggdrop/scripts/" <-- the last / is IMPORTANT! 
 
set inifile "repeatkick.ini"
set tempfile "$scriptdir$inifile"
set inifile $tempfile
unset tempfile 

set scriptfile "repeatkick.tcl"
set tempfile "$scriptdir$scriptfile"
set scriptfile $tempfile
unset tempfile

set logdirtemp "rklogs"
set logdir "$scriptdir$logdirtemp"
unset logdirtemp

#---------------------------------------------------------------------------------------------
# BINDINGS
#---------------------------------------------------------------------------------------------
bind pubm - * repeat_pubm
bind ctcp - ACTION repeat_action
bind nick - * repeat_nick
bind msg o|o "repeatchk" repeat_check
bind msg o|o "repeatvars" repeat_vars
bind msg o|o "repeatinfo" repeat_info
bind msg n "repeatset" repeat_set
bind join - * check_banned
bind flud - * flood_check
bind dcc m rkdebug debug_mode
bind dcc m rklink bot_linking

#---------------------------------------------------------------------------------------------
# DCC COMMANDS
#---------------------------------------------------------------------------------------------
proc debug_mode {hand idx arg} {
   global debug_mode
   if {$debug_mode == 0} {
      putlog "-- RK: Now running in DEBUG mode. (set by $hand)"
      set debug_mode 1
   } else {
      putlog "-- RK: No longer running in DEBUG mode. (cancelled by $hand)"
      set debug_mode 0
   }
}

proc bot_linking {hand idx arg} {
   global RKlinking botnick
   if {$RKlinking == 0} {
      putlog "-- RK: $hand ENABLED RK botlinking on bot $botnick."
      set RKlinking 1
      bind bot - iamhandling bot_being_handled
      bind bot - closehandle bot_close_handle
      bind bot - filesync bot_filesync
      bind bot - whoishere bot_respond_usage
      bind bot - iamhere bot_using_script
      bind bot - wantfile bot_wantsfile
      bind bot - ackreload bot_ackreload
      bind bot - valuechange bot_valuechange
      bind bot - timercheck bot_timercheck
      bind bot - timerrunning bot_timerrunning
      bind bot - inisettings bot_inireceive  
      bind bot - inifinished bot_inifinished
      bind rcvd b * bot_file_received
      bind link - * bot_single_check_usage
      bot_check_usage
   } else {
      putlog "-- RK: $hand DISABLED RK botlinking on bot $botnick"
      set RKlinking 0
      unbind bot - iamhandling bot_being_handled
      unbind bot - closehandle bot_close_handle
      unbind bot - filesync bot_filesync
      unbind bot - whoishere bot_respond_usage
      unbind bot - iamhere bot_using_script
      unbind bot - wantfile bot_wantsfile
      unbind bot - ackreload bot_ackreload
      unbind bot - valuechange bot_valuechange
      unbind bot - timercheck bot_timercheck
      unbind bot - timerrunning bot_timerrunning
      unbind bot - inisettings bot_inireceive
      unbind bot - inifinished bot_inifinished
      unbind rcvd b * bot_file_received
      unbind link - * bot_single_check_usage
   }
}


#---------------------------------------------------------------------------------------------
# .INI FILE HANDLING
#---------------------------------------------------------------------------------------------
proc parse_ini {} {
   global inifile repeatkick repeattime repeatmsg repeatban repeatbantime repeatonoff repeattimer floodmsg floodonoff chartolerance postolerance logging
   # Read settings for repeatkick (if file exists)
   if {![file exists $inifile] || [file size $inifile] == 0} {
      putlog "-- RK: .ini file zero length, missing or invalid.  Using defaults."
      create_ini "new"
      return
   } \
   else {
      # .ini file exists.  Process it.
      set ctr 0
      set fd [open "$inifile" r]
      putlog "-- RK: .INI file found.  Processing ($fd)"
      while { ![eof $fd] } {
         gets $fd buffer($ctr)
         incr ctr
      }
      close $fd
      set ctr [expr $ctr - 1]

      # Check to see number of lines in file corresponds to what we expect
      # ie. for each chan the bot is on, need 11 lines (11 variables + placeholder)
      if {$ctr != [expr [llength [channels]] * 13]} {
         # Eg.  Bot is on 4 channels, if $ctr is not equal to 4 * 13 (52) then...
         putlog "-- RK: Error(s) in .INI file.  Expected [expr [llength [channels]] * 13] lines, but found $ctr"
         create_ini "new"
         return
      }

      set chanadd ""
      for {set counter 0} {$counter < $ctr} {set counter [expr $counter+13]} {
         set chan [string tolower $buffer($counter)]
         set repeatkick($chan) $buffer([expr $counter + 1])
         set repeattime($chan) $buffer([expr $counter + 2])
         set repeatmsg($chan) $buffer([expr $counter + 3])
         set floodmsg($chan) $buffer([expr $counter + 4]) 
         set repeatban($chan) $buffer([expr $counter + 5])
         set repeatbantime($chan) $buffer([expr $counter + 6])
         set repeatonoff($chan) $buffer([expr $counter + 7])
         set floodonoff($chan) $buffer([expr $counter + 8])
         set chartolerance($chan) $buffer([expr $counter + 9])
         set postolerance($chan) $buffer([expr $counter + 10])
         set logging($chan) $buffer([expr $counter + 11])
         set repeattimer($chan) 0
         set chanadd "$chanadd, $chan"    
      }
      set chanadd [string range $chanadd 2 [expr [string length $chanadd] - 1]]
      putlog "-- RK: $chanadd - OK."
      putlog "-- RK: .INI file parsed successfully."
   }
}

proc create_ini {type} {
   global inifile repeatkick repeattime repeatmsg repeatban repeatbantime repeatonoff repeattimer floodmsg floodonoff chartolerance postolerance logging
   # Create a new .INI file if the current one is missing/corrupt
   set fd [open "$inifile" w]
   foreach chan [channels] {
      puts $fd $chan
      puts $fd $repeatkick($chan)
      puts $fd $repeattime($chan)
      puts $fd $repeatmsg($chan)
      puts $fd $floodmsg($chan)
      puts $fd $repeatban($chan)
      puts $fd $repeatbantime($chan)
      puts $fd $repeatonoff($chan)
      puts $fd $floodonoff($chan)
      puts $fd $chartolerance($chan)
      puts $fd $postolerance($chan)
      puts $fd $logging($chan) 
      # Placeholder
      puts $fd "----------------------------------------------"
   }
   if {$type == "new"} {putlog "-- RK: New .INI file created successfully ($fd)."} else {putlog "-- RK: .INI file amended OK ($fd)."}
   close $fd
}

#---------------------------------------------------------------------------------------------
# BOTLINKING functions (used in linked Repeatkicking bots)
#---------------------------------------------------------------------------------------------
bind bot - iamhandling bot_being_handled
bind bot - closehandle bot_close_handle
bind bot - filesync bot_filesync
bind bot - whoishere bot_respond_usage
bind bot - iamhere bot_using_script
bind bot - wantfile bot_wantsfile
bind bot - ackreload bot_ackreload
bind bot - valuechange bot_valuechange
bind bot - timercheck bot_timercheck
bind bot - timerrunning bot_timerrunning
bind bot - inisettings bot_inireceive
bind bot - inifinished bot_inifinished
bind rcvd b * bot_file_received
bind link - * bot_single_check_usage 

proc bot_being_handled {bot cmd arg} {
   global bot_handling debug_mode
   set type [lindex $arg 0]
   # Action is being dealt with by another bot.
   set bot_handling 1
   if {$debug_mode == 1} {
      if {$type == 0} {putlog "$bot --> doing flood & repeat pub/action check"} else {putlog "$bot --> doing repeat/floodkick punishment"}
   }
   if {$type == 1} {putlog "-- RK: Detected repeat/flood.  $bot is handling."}
}  

proc bot_close_handle {bot cmd arg} {
   global bot_handling
   # Action has been completed.
   set bot_handling 0
}  

proc get_filesize {file} {
   # Gets TRUE file size.  (Note that [file size $file] returns a different value
   # for UNIX systems as opposed to Windows systems.  This fixes that problem.

   set tt [open "$file" r]
   set buffer [read $tt] 
   return [string length $buffer]
}

proc bot_timercheck {bot cmd arg} {
   # Another bot is asking me if Im running a timer, and what the current time is.
   global repeattimer

   # Timerval - 0 = repeatkick clean timer
   #            1 = channel timer
   set timerval [lindex $arg 0]
   set chan [lindex $arg 1]

   switch -exact $timerval {
      0 { if {[regexp repeatk_clean [timers]]} {
              # Get timer value.
              foreach timr [timers] {
                 if {[lindex $timr 1] == "repeatk_clean"} { set rkclean_time [lindex $timr 0] }
              }
              putbot $bot "timerrunning $rkclean_time 0"
              unset rkclean_time
          } else { putbot $bot "timerrunning 0 0" }
        }
      1 {  # Does timer exist in [utimers] ?
           if {(![validchan $chan]) || ([string match "*$repeattimer($chan)*" [utimers]] == 0)} {
              # Timer doesn't exist (bot isn't on the quoted channel)
              putbot $bot "timerrunning 0 $chan"
           } else {
              # Find the timer. 
              foreach timr [utimers] {
                 if {[lindex $timr 2] == $repeattimer($chan)} { set rkchan_time [lindex $timr 0] }
              }
              putbot $bot "timerrunning $rkchan_time $chan"
              unset rkchan_time
           }
        }
    } 
}

proc bot_timerrunning {bot cmd arg} {
  # Called when a bot has sent me information on a timer.
  global repeattimer repeattime
  set timerval [lindex $arg 0]
  set chan [lindex $arg 1]

  if {$chan == 0} {
     if {$timerval != 0} {
        # Bot is already running a timer, use that one.
        # (Kill existing timer.)
        foreach timr [timers] {
           if {[lindex $timr 1] == "repeatk_clean"} { 
              if {[lindex $timr 0] == $timerval} {return}
              set mytimer [lindex $timr 0]
              killtimer [lindex $timr 2] 
              putlog "-- RK: Resyncing 'repeatk_clean' timer (old: $mytimer mins, new: $timerval mins)"
           }
        }
        # .. and start a new one at the passed value.
        timer $timerval repeatk_clean
     }
  } else {
     # Passed a channel..
     if {$timerval != 0} {
        # Bot is already running a timer, use that one.
        # (Kill existing timer.)
        foreach timr [utimers] {
           if {[lindex $timr 2] == $repeattimer($chan)} { 
              if {[lindex $timr 0] == $timerval} {return}
              set mytimer [lindex $timr 0]
              killutimer $repeattimer($chan) 
              putlog "-- RK: Resyncing '$chan' timer (old: $mytimer secs, new: $timerval secs)"
           }
        }
        # .. and start a new one at the passed value.
        set repeattimer($chan) [utimer $timerval repeat_timr]      
     }  
  } 
}

proc bot_filesync {bot cmd arg} {
   set inifsize [lindex $arg 1]
   set scriptfsize [lindex $arg 2]
   set inif_lastm [lindex $arg 3]
   set scriptf_lastm [lindex $arg 4]

   global botnick inifile scriptfile repeatkick repeattime repeatmsg repeatban repeatbantime repeatonoff repeattimer floodmsg floodonoff chartolerance postolerance logging inittime inichanged
   # This procedure is used for file resync.
   # The bot will request a file resync with any bot that acknowledges running this
   # script.  It is used only when a bot first rehashes, as all other syncs are done
   # on the fly during normal operation.

   putlog "-- RK: Syncing RK files with '$bot'."
   if {[file exists $scriptfile] && [file exists $inifile]} {
      set myinifsize [get_filesize $inifile]
      set myscriptfsize [get_filesize $scriptfile]
      set myinif_lastm [get_filesize $inifile]
      set myscriptf_lastm [get_filesize $scriptfile]   
   }  
 
   # Request files..
   # Files are assumed to be older if:
   # a) they are not equal in size (can be larger or smaller)
   # b) the last modified time is earlier (or indeed later)

   # ------------------------------------------------------------------
   # SCRIPT FILE SHARING
   # The whole file is DCC'd to whichever bot has the older/smaller
   # version.  There are no 'in-file' examinations made.
   # ------------------------------------------------------------------
   if {![file exists $scriptfile]} {
      # Script file doesn't even exist.
      putlog "-- RK: Script file missing, invalid or zero-length.  Getting $bot's"
      putbot $bot "wantfile $scriptfile 0"
   } elseif {($scriptf_lastm > $myscriptf_lastm) && ($myscriptfsize != $scriptfsize)} {
      # Other bot has newer version of script.  Request it.
      putlog "-- RK: $bot's script file is newer ($scriptf_lastm - [clock format $scriptf_lastm -format "%d/%m/%y"]), $scriptfsize bytes) (my version is $myscriptf_lastm - [clock format $myscriptf_lastm -format "%d/%m/%y"]), $myscriptfsize bytes)"
      putlog "-- RK: Requesting newer copy of script file from $bot"
      putbot $bot "wantfile $scriptfile 0"
   } elseif {($scriptf_lastm < $myscriptf_lastm) && ($myscriptfsize != $scriptfsize)} {
      # I have newer version, send it to other bot.
      putlog "-- RK: $bot's script file is older ($scriptf_lastm ([clock format $scriptf_lastm -format "%d/%m/%y"]), $scriptfsize bytes) (my version is $myscriptf_lastm - [clock format $myscriptf_lastm -format "%d/%m/%y"]), $myscriptfsize bytes)"
      putlog "-- RK: Attempting send of newer script file to $bot"
      dccsend $scriptfile $bot
   } else {
      # They have the current version!.. Yay!
      putlog "-- RK: $bot is using the same script file."
   }

   # ------------------------------------------------------------------
   # INI FILE SHARING
   # As opposed to copying the whole file across from one bot to
   # another, the bots tell eachother what channels they are on and
   # the relevant 'repeatkick' settings.
   # ------------------------------------------------------------------
   foreach chanon [channels] {
      putbot $bot "inisettings $chanon $repeatkick($chanon) $repeattime($chanon) $repeatmsg($chanon) $repeatban($chanon) $repeatbantime($chanon) $repeatonoff($chanon) $floodmsg($chanon) $floodonoff($chanon) $chartolerance($chanon) $postolerance($chanon) $logging($chanon) $inittime"
   }
   # Tell bot to write out INI file
   if {$inichanged == 1} {
      putbot $bot "inifinished"
      set inichanged 0
   }
}

proc bot_inireceive {bot cmd arg} {
   global botnick inifile scriptfile repeatkick repeattime repeatmsg repeatban repeatbantime repeatonoff floodmsg floodonoff chartolerance postolerance logging inittime inichanged

   set chan [lindex $arg 0]
   set botinittime [lindex $arg 13]

   # Just EXIT if Im not on one of the other bot's channels,
   if {![validchan $chan]} {return}

   # ..or this bot was started before the other (ie. ignore bouncebacks)
   if {($inittime < $botinittime) && $botinittime != ""} {return}

   # If incorrect number of variables passed, return.
   if {[llength $arg] != 13} {
      putlog "-- RK: $bot passed [llength $arg] vars to me ($chan), expected 13."
      return
   }

   for {set ctr 0} {$ctr <= [llength $arg]} {incr ctr} {
      # Buffer received values
      set buffer($ctr) [lindex $arg $ctr]
   }
    
   set outtext "Changes ($chan):"
   set outtext2 $outtext
   # ------------------------------------------------------------------
   # VARIABLE COMPARISON CHECKS
   # ------------------------------------------------------------------
   # ------------------------------------------------------------------
   # REPEATKICK variable (passed value: 1)
   # ------------------------------------------------------------------
   if {[string compare $buffer(1) $repeatkick($chan)] != 0} {
      set outtext "$outtext 'repeatkick' (was: $repeatkick($chan), now: $buffer(1)), "
      set repeatkick($chan) $buffer(1)
   }
   # ------------------------------------------------------------------
   # REPEATTIME variable (passed value: 2)
   # ------------------------------------------------------------------
   if {[string compare $buffer(2) $repeattime($chan)] != 0} {
      set outtext "$outtext 'repeattime' (was: $repeattime($chan), now: $buffer(2)), "
      set repeattime($chan) $buffer(2)
   }
   # ------------------------------------------------------------------
   # REPEATMSG variable (passed value: 3)
   # ------------------------------------------------------------------
   if {[string compare $buffer(3) $repeatmsg($chan)] != 0} {
      set outtext "$outtext 'repeatmsg' (was: $repeatmsg($chan), now: $buffer(3)), "
      set repeatmsg($chan) $buffer(3)
   }
   # ------------------------------------------------------------------
   # REPEATBAN variable (passed value: 4)
   # ------------------------------------------------------------------
   if {[string compare $buffer(4) $repeatban($chan)] != 0} {
      set outtext "$outtext 'repeatban' (was: $repeatban($chan), now: $buffer(4)), "
      set repeatban($chan) $buffer(4)
   }
   # ------------------------------------------------------------------
   # REPEATBANTIME variable (passed value: 5)
   # ------------------------------------------------------------------
   if {[string compare $buffer(5) $repeatbantime($chan)] != 0} {
      set outtext "$outtext 'repeatbantime' (was: $repeatbantime($chan), now: $buffer(5)), "
      set repeatbantime($chan) $buffer(5) 
   }
   # ------------------------------------------------------------------
   # REPEATONOFF variable (passed value: 6)
   # ------------------------------------------------------------------
   if {[string compare $buffer(6) $repeatonoff($chan)] != 0} {
      set outtext "$outtext 'repeatonoff' (was: $repeatonoff($chan), now: $buffer(6)), "
      set repeatonoff($chan) $buffer(6)
   }
   # ------------------------------------------------------------------
   # FLOODMSG variable  (passed value: 7)
   # ------------------------------------------------------------------
   if {[string compare $buffer(7) $floodmsg($chan)] != 0} {
      set outtext "$outtext 'floodmsg' (was: $floodmsg($chan), now: $buffer(7)), "
      set floodmsg($chan) $buffer(7)
   }
   # ------------------------------------------------------------------
   # FLOODONOFF variable  (passed value: 8)
   # ------------------------------------------------------------------
   if {[string compare $buffer(8) $floodonoff($chan)] != 0} {
      set outtext "$outtext 'floodonoff' (was: $floodonoff($chan), now: $buffer(8)), "
      set floodonoff($chan) $buffer(8)
   }
   # ------------------------------------------------------------------
   # CHARTOLERANCE variable  (passed value: 9)
   # ------------------------------------------------------------------
   if {[string compare $buffer(9) $chartolerance($chan)] != 0} {
      set outtext "$outtext 'chartolerance' (was: $chartolerance($chan), now: $buffer(9)), "
      set chartolerance($chan) $buffer(9)
   }
   # ------------------------------------------------------------------
   # POSTOLERANCE variable  (passed value: 10)
   # ------------------------------------------------------------------
   if {[string compare $buffer(10) $postolerance($chan)] != 0} {
      set outtext "$outtext 'postolerance' (was: $postolerance($chan), now: $buffer(10)), "
      set postolerance($chan) $buffer(10)
   }
   # ------------------------------------------------------------------
   # LOGGING variable  (passed value: 11)
   # ------------------------------------------------------------------
   if {[string compare $buffer(11) $logging($chan)] != 0} {
      set outtext "$outtext 'logging' (was: $logging($chan), now: $buffer(11)), "
      set logging($chan) $buffer(11)
   }
 
   # Inform user of changes.
   if {[string compare $outtext $outtext2] != 0} {
      # Changes made.
      set outtext [string range $outtext 0 [expr [string length $outtext] - 3]]
      putlog "-- RK: $outtext"
      set inichanged 1
   }
   unset outtext
   unset outtext2 
   unset buffer
}
      
proc bot_inifinished {bot cmd arg} {
   # Write out the amended INI file.
   # Called when changes have been brought over from another bot.
   create_ini "amend"
}

proc bot_wantsfile {bot cmd arg} {
   global botnick inifile scriptfile openfn
   set file [lindex $arg 0]
   set type [lindex $arg 1]
   
   if {$type == 0} {
      putlog "-- RK: $bot wants my newer copy of script file.  Sending."
      set tempfile $scriptfile
   } elseif {$type == 1} {
      putlog "-- RK: $bot wants my newer copy of INI file.  Sending."
      set tempfile $inifile
   }
   dccsend $tempfile $bot
}

proc bot_reloadscript {type} {
   global inifile scriptfile botnick bot_scriptrunner scriptdir

   foreach bot [bots] {
      if {$bot_scriptrunner($bot) == 1} {putbot $bot "ackreload $botnick $type"}
   }

   if {$type == 0} {
      putlog "-- RK: Reloading RK script file."
      source $scriptfile
   } elseif {$type == 1} {
      putlog "-- RK: Reloading INI file."
      parse_ini
   }
}  

proc bot_ackreload {bot cmd arg} {
   global RKlinking
   set type [lindex $arg 1]
   putlog "-- RK: $bot acknowledged end of burst."
   if {$type == 0} {putlog "-- RK: $bot has reloaded script successfully."} else {putlog "-- RK: $bot has reloaded INI file successfully."}
} 

proc bot_respond_usage {bot cmd arg} {
   global botnick
   # Called when another bot is asking for script runners to identify themselves.
   putbot $bot "iamhere $botnick"
   putlog "-- RK: $bot has queried me for RK authentication.."
}

proc bot_using_script {bot cmd arg} {
   global botnick inifile scriptfile bot_scriptrunner version_num
   # When 'iamhere' is received by the bot, the bot that sent it is added to the list
   # of valid scriptrunners. 
   set bot_scriptrunner($bot) 1
   putlog "-- RK: $bot acknowledged running Repeatkick $version_num."
      
   # Request resync
   putbot $bot "filesync $botnick [get_filesize $inifile] [get_filesize $scriptfile] [file mtime $inifile] [file mtime $scriptfile]" 
   putlog "-- RK: Requesting RK filesync with $bot."

   # Sync repeatkick timers
   putbot $bot "timercheck 0 0"
   foreach chan [channels] {
      putbot $bot "timercheck 1 $chan"
   }
} 

proc bot_file_received {hand nick pathtofile} {
  # This function is called when the file is successfully received by the bot.
  global inifile scriptfile botnick scriptdir
  if {![matchattr $hand b]} { return }  
  if {[string compare [string tolower [file tail $pathtofile]] [string tolower [file tail $scriptfile]]] == 0} { set type 0 } \
     elseif {[string compare [string tolower [file tail $pathtofile]] [string tolower [file tail $inifile]]] == 0} { set type 1 } \
     else {return 0}

  if {$type == 1} {set result [ catch { file copy -force $pathtofile $inifile } ]} \
     elseif {$type == 0} {set result [ catch { file copy -force $pathtofile $scriptfile } ]} 
  if {$result} {
     putlog "-- RK: ERROR occured during DCC transmission."
     return 0
  } else {
     putlog "-- RK: Received updated file from $hand OK ([get_filesize $pathtofile] bytes)."
     putlog "-- RK: Temporary file: $pathtofile"
     if {$type == 1} {putlog "-- RK: '[file tail $pathtofile]' copied as '$inifile' OK"} else {putlog "-- RK: '[file tail $pathtofile]' copied as '$scriptfile' OK"}
     set result [ catch { file delete -force $pathtofile } ]
     if {$result} {
        putlog "-- RK: ERROR occured during attempted deletion of temporary file."
     } else {
        putlog "-- RK: '[file tail $pathtofile]' deleted from 'incoming' dir."
     }
     bot_reloadscript $type
  }  
}
   
proc bot_check_usage {} {
   global botnick
   # Send out a request for all other bots running this script to make themselves known.
   putallbots "whoishere $botnick"
   set bottys 0
   foreach bot [bots] {
      incr bottys
   }
   putlog "-- RK: Sent request to $bottys BOTNET bots to identify."
   unset bottys
}

proc bot_single_check_usage {bot via} {
   global botnick bot_scriptrunner
   # Send out an individual request to newly linked bot.
   set bot_scriptrunner($bot) 0
   putlog "-- RK: Sent request to $bot to identify."
   putbot $bot "whoishere $botnick"
}

proc bot_valuechange {bot cmd arg} {
   global inifile repeatkick repeattime repeatmsg repeatban repeatbantime repeatonoff repeattimer floodmsg floodonoff chartolerance postolerance logging

   set variable [lindex $arg 1]
   set chan [lindex $arg 2]
   set value [lindex $arg 3]

   # This is called when a change is made to a bot's INI file.
   # Used for on-the-fly resyncing.
   putlog "-- RK: $bot requested sync'ing for var. '$variable' on $chan, now $value."
   switch -exact $variable {
      repeatkick {set repeatkick($chan) $value}
      repeattime {set repeattime($chan) $value}
      repeatmsg {set repeatmsg($chan) $value}
      floodmsg {set floodmsg($chan) $value}
      repeatban {set repeatban($chan) $value}
      repeatbantime {set repeatbantime($chan) $value}
      repeatonoff {set repeatonoff($chan) $value}
      floodonoff {set floodonoff($chan) $value}
      chartolerance {set chartolerance($chan) $value}
      postolerance {set postolerance($chan) $value}
      logging {set logging($chan) $value}
   }   
   create_ini "amend"
}

#---------------------------------------------------------------------------------------------
# BAN CHECK procedure
# This forms part of the 'intelligent' banning that Repeatkick does.  Basically when a new
# user joins the channel, the bot evaluates whether or not this person has evaded a previously
# set ban (easy to do if they are on a dialup host).
#
# It checks for 6 things, which if any return TRUE, it is assumed joining user == banned user.
# -- 1) 'Joined nick' is the same as 'banned nick'
# -- 2) 'Joined ident' is the same as 'banned ident'
# -- 3) 'Joined nick' fails MASK checking on 'banned nick' (see procedure mask_check)
# -- 4) 'Joined ident' fails MASK checking on 'banned ident' (see procedure mask_check)
# -- 5) 'Joined nick' fails LETTER checking on 'banned nick' (see procedure letter_check)
# -- 6) 'Joined ident' fails LETTER checking on 'banned ident' (see procedure letter_check)
#---------------------------------------------------------------------------------------------
proc check_banned {nick host hand chan} {
   global repeat_last repeat_num repeatkick repeatmsg repeatban repeatbantime floodmsg bot_handling bot_scriptrunner botnick RKlinking inuse timercalled

   set inuse 1

   # Check to see if they match an ident entry in the database..
   foreach nortyuser [userlist] {
      if { [matchattr $nortyuser D] } {
         set badnick [getuser $nortyuser XTRA repeat_nick]
         set badident [getuser $nortyuser XTRA repeat_ident]
         set badhost [getchanhost $nick $chan]
         set joinedid [string range $badhost 0 [expr [string first "@" $badhost] - 1]]
         set oldhost [getuser $nortyuser XTRA repeat_oldhst]
         set n [string tolower $nick]         

         # -----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
         # -------------------- IF NICK IS THE SAME AS BEFORE ----------------------- OR ------------------------ IDENT IS THE SAME AS BEFORE ----------------------- OR ------- NICK FAILS MASK CHECK ------ OR --------------------------- IDENT FAILS MASK CHECK -------------------------- OR -------- NICK FAILS LETTER CHECK ---------- OR -------------------------------- IDENT FAILS LETTER CHECK -----------------------------
         # -----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
         if { ([string compare [string tolower $nick] [string tolower $badnick]] == 0 || [string compare [string tolower $badident] [string tolower $joinedid]] == 0) || ([mask_check $n $nick $badnick] == 1 || ([mask_check $n $joinedid $badident] == 1 && [string length $badident] >= 4)) || ([letter_check $n $n $chan $badnick 0 $oldhost] == 1 || ([letter_check $n $joinedid $chan $badident 0 $oldhost] == 1 && [string length $badident] >= 4))} {
            # Ban outstanding?.. if it is, then they've connected on a different host.
            if { [isban $oldhost $chan] } {

               # Before doing anything, check to see that another bot isn't handling this already..
               # If so, break out.  If not, lock it so I am doing it.
               if {$RKlinking == 1} {
                  if {$bot_handling == 1} {
                     set bot_handling 0
                     return
                  } else {
                     foreach bot [bots] {
                        if {$bot_scriptrunner($bot) == 1} {putbot $bot "iamhandling"}
                     }
                  }
               }

               # The next line is used just in case the person rejoins before the ban is set.
               if { ![string compare [string tolower "*!*[string range $badhost [string first "@" $badhost] end]"] [string tolower $oldhost]] == 0 } {

                  # Refresh bantime by renewing ban on new host, and removing it on the old one.
                  putflog $chan "-- RK: '$badnick' (banned) has rejoined under a different host. ($host)"
                  set banmask "*!*[string range $badhost [string first "@" $badhost] end]"
                  newchanban $chan $banmask repeat Repeating/Flooding $repeatbantime($chan)
                  putflog $chan "-- RK: Added new ban for '$nick' ($host).. OK"
                  killchanban $chan $oldhost
                  putflog $chan "-- RK: Removed ban: ($oldhost).. OK"
                  setuser $nortyuser XTRA repeat_oldhst $banmask
                  setuser $nortyuser XTRA repeat_num [expr [getuser $nortyuser XTRA repeat_num] + 1] 
               }
               putserv "KICK $chan $nick :Re-banned: Earlier repeat/flood on $chan. ($badnick)"

               if {$RKlinking == 1} {
                  # Clear RK handling locks
                  set bot_handling 0
                  foreach bot [bots] {
                    if {$bot_scriptrunner($bot) == 1} {putbot $bot "closehandle"}
                  }
               }
            }
         }
      }
   }

   # Arrays can now be cleared if need be.
   if {$timercalled == 1} {
      catch {unset repeat_last}
      catch {unset repeat_num}
      set timercalled 0
   }
   set inuse 0
}

#---------------------------------------------------------------------------------------------
# Repeatcheck procedure (called when user requests a list of repeatkicks logged)
# Syntax: /msg <bot> repeatcheck
#---------------------------------------------------------------------------------------------
proc repeat_check {nick host hand arg} {
   # List today's offenders :)
   putlog "-- RK: ($nick) requested Repeatkick list." 
   putserv "NOTICE $nick :Repeatkicks issued as of [ctime [unixtime]]:"
   foreach timex [timers] {
      if { [lindex $timex 1] == "repeatk_clean" } {
         set time_left [lindex $timex 0]
      }
   }  
   foreach nortyuser [userlist] {
      if { [matchattr $nortyuser D] } {
         # Found one
         set badnick [getuser $nortyuser XTRA repeat_nick]
         set badident [getuser $nortyuser XTRA repeat_ident]
         if { [getuser $nortyuser XTRA repeat_num] == 1 } {
            putserv "NOTICE $nick :  1$badnick ($badident)  ..41 1warning."
         }
         if { [getuser $nortyuser XTRA repeat_num] == 2 } {
            putserv "NOTICE $nick :  1$badnick ($badident)  ..42 1warnings."
         }
         if { [getuser $nortyuser XTRA repeat_num] >= 3 } {
            set no_of_bans [expr [getuser $nortyuser XTRA repeat_num] - 2]
            putserv "NOTICE $nick :  1$badnick ($badident)  ..BANNED 4$no_of_bans 1times."
         }
      }    
   }
   putserv "NOTICE $nick :Next DB cleanup: $time_left minutes"
   putserv "NOTICE $nick :-- End of List --"
}

#---------------------------------------------------------------------------------------------
# Repeat variables procedure (called when user requests a list of repeat variables stored)
# Syntax: /msg <bot> repeatvars
#---------------------------------------------------------------------------------------------
proc repeat_vars {nick host hand arg} {
   # Give some variables being stored.
   putlog "-- RK: ($nick) requested Repeatkick variables." 
   putserv "NOTICE $nick :Repeatkick: Users/Variables. (for debugging purposes)"
   foreach nortyuser [userlist] {
     if { [matchattr $nortyuser D] } {
        # Found one
        set os "1[getuser $nortyuser XTRA repeat_nick]:"
        set os "$os repeat_num: 12[getuser $nortyuser XTRA repeat_num],"
        set os "$os repeat_nick: 12[getuser $nortyuser XTRA repeat_nick],"
        set os "$os repeat_ident: 12[getuser $nortyuser XTRA repeat_ident]" 
        set os "$os repeat_host: 12[getuser $nortyuser XTRA repeat_oldhst]"
        putserv "NOTICE $nick :$os"
     }
   }        
}

#---------------------------------------------------------------------------------------------
# Repeat info procedure (called when a user wants info on channel RK settings)
# Syntax: /msg <bot> repeatinfo <#channel>
#---------------------------------------------------------------------------------------------
proc repeat_info {nick host hand arg} {
   # Some channel settings for repeatkick.
   putlog "-- RK: ($nick) requested Repeatkick channel info."
   global botnick repeatkick repeattime repeatmsg repeatban repeatbantime repeatonoff repeattimer floodmsg floodonoff chartolerance postolerance logging
   set chan [lindex $arg 0]
   if {$chan == ""} {
      putserv "NOTICE $nick :Usage: /msg $botnick repeatinfo <channel>"
      putserv "NOTICE $nick :   - gives Repeatkick channel setup information."
      return
   }

   if {![validchan $chan]} {
      putserv "NOTICE $nick :$chan is not a valid channel (ie. one I am monitoring)"
      return
   }
   putserv "NOTICE $nick :$chan settings:"
   putserv "NOTICE $nick :-- Kick on (n) repeats (repeatkick): $repeatkick($chan)"
   putserv "NOTICE $nick :-- In how many seconds (repeattime): $repeattime($chan)"     
   putserv "NOTICE $nick :-- Repeatkick'ing msg (repeatmsg): $repeatmsg($chan)"
   putserv "NOTICE $nick :-- Floodkick'ing msg (floodmsg): $floodmsg($chan)"
   if {$repeatban($chan) == 0} { putserv "NOTICE $nick :-- Ban on any repeat? (repeatban): No (0)" } \
      elseif {$repeatban($chan) == 1} { putserv "NOTICE $nick :-- Ban on any repeat? (repeatban): Yes (1)" }
   putserv "NOTICE $nick :-- 'Char. presence' tolerance (%) (chartolerance): $chartolerance($chan)%"
   putserv "NOTICE $nick :-- 'Positional match' tolerance (%) (postolerance): $postolerance($chan)%"
   putserv "NOTICE $nick :-- Repeatkick ban time (repeatbantime): $repeatbantime($chan)"
   if {$repeatonoff($chan) == 0} { putserv "NOTICE $nick :-- Repeatkick enabled? (repeatonoff): No (0)" } \
      elseif {$repeatonoff($chan) == 1} { putserv "NOTICE $nick :-- Repeatkick enabled? (repeatonoff): Yes (1)" }
   if {$floodonoff($chan) == 0} { putserv "NOTICE $nick :-- Floodkick enabled? (floodonoff): No (0)" } \
      elseif {$floodonoff($chan) == 1} { putserv "NOTICE $nick :-- Floodkick enabled? (floodonoff): Yes (1)" }
   if {$logging($chan) == 0} { putserv "NOTICE $nick :-- Logging enabled? (logging): No (0)" } \
      elseif {$logging($chan) == 1} { putserv "NOTICE $nick :-- Logging enabled? (logging): Yes (1)" }
   putserv "NOTICE $nick :-- Timer ID: $repeattimer($chan)"
}

#---------------------------------------------------------------------------------------------
# Repeat set procedure (called when user wishes to change stored channel settings)
# Syntax: /msg <bot> repeatset <#channel> <variable> [value]
#---------------------------------------------------------------------------------------------
proc repeat_set {nick host hand arg} {
   # Set channel Repeatkick settings
   global botnick repeatkick repeattime repeatmsg repeatban repeatbantime repeatonoff repeattimer floodmsg floodonoff bot_scriptrunner chartolerance postolerance logging

   set chan [string tolower [lindex $arg 0]]
   set var [string tolower [lindex $arg 1]]
   set value [lindex $arg 2]
 
   if {![validchan $chan]} {
      putserv "NOTICE $nick :$chan is not a valid channel (ie. one I am monitoring)"
      return
   }

   switch -exact $var {
      repeatkick {
         # ------------------------------------------------------------------
         # REPEATKICK variable
         # ------------------------------------------------------------------
         if { $value == "" } { 
            putserv "NOTICE $nick :Usage: /msg $botnick repeatset <#channel> repeatkick <new value>"
            return 
         }
         if { $value == $repeatkick($chan) } { 
            putserv "NOTICE $nick :Setting for 'repeatkick' on $chan is already $value."
            return 
         }
         if { $value <= 0 || $value > 10 } { 
            putserv "NOTICE $nick :Value must be between 1 and 10."
            return 
         }
         set repeatkick($chan) $value
         foreach bot [bots] {
            if {$bot_scriptrunner($bot) == 1} {putbot $bot "valuechange $botnick repeatkick $chan $value"}
         }
         putserv "NOTICE $nick :'repeatkick' changed to '$value' for $chan."
         putflog $chan "-- RK: ($nick) changed 'repeatkick' for $chan to: $value."
      }
      repeattime {
         # ------------------------------------------------------------------
         # REPEATTIME variable
         # ------------------------------------------------------------------
         if { $value == "" } {
            putserv "NOTICE $nick :Usage: /msg $botnick repeatset <#channel> repeattime <new value>"
            return 
         }
         if { $value == $repeattime($chan) } { 
            putserv "NOTICE $nick :Setting for 'repeattime' on $chan is already $value." 
            return 
         }
         if {$value <= 0 || $value > 60} { 
            putserv "NOTICE $nick :Value must be between 1 and 60."
            return 
         }
         set repeattime($chan) $value
         foreach bot [bots] {
            if {$bot_scriptrunner($bot) == 1} {putbot $bot "valuechange $botnick repeattime $chan $value"}
         }
         putserv "NOTICE $nick :'repeattime' changed to '$value' for $chan."
         putflog $chan "-- RK: ($nick) changed 'repeattime' for $chan to: $value."

         # Kill existing timer, and start a new one.
         if {[string match "*$repeattimer($chan)*" [utimers]] == 0} {
            set repeattimer($chan) [utimer $repeattime($chan) repeat_timr]
         }        
      }
      repeatmsg {
         # ------------------------------------------------------------------
         # REPEATMSG variable
         # ------------------------------------------------------------------
         if { $value == "" } { 
            putserv "NOTICE $nick :Usage: /msg $botnick repeatset <#channel> repeatmsg <kick reason>"
            return 
         }
         if { [string compare [string tolower $value] $repeatmsg($chan)] == 0 } { 
            putserv "NOTICE $nick :'repeatmsg' is already set to '$value'"
            return 
         }
         if {[string length $value] > 40} { 
            putserv "NOTICE $nick :'repeatmsg' can be no longer than 40 characters."
            return 
         }
         set repeatmsg($chan) $value
         foreach bot [bots] {
            if {$bot_scriptrunner($bot) == 1} {putbot $bot "valuechange $botnick repeatmsg $chan $value"}
         }
         putserv "NOTICE $nick :'repeatmsg' changed to '$value' for $chan."
         putflog $chan "-- RK: ($nick) changed 'repeatmsg' for $chan to: $value."
      }
      floodmsg {
         # ------------------------------------------------------------------
         # FLOODMSG variable
         # ------------------------------------------------------------------
         if { $value == "" } { 
            putserv "NOTICE $nick :Usage: /msg $botnick repeatset <#channel> floodmsg <kick reason>"
            return 
         }
         if { [string compare [string tolower $value] $floodmsg($chan)] == 0 } { 
            putserv "NOTICE $nick :'floodmsg' is already set to '$value'"
            return 
         }
         if {[string length $value] > 40} { 
            putserv "NOTICE $nick :'floodmsg' can be no longer than 40 characters."
            return 
         }
         set floodmsg($chan) $value
         foreach bot [bots] {
            if {$bot_scriptrunner($bot) == 1} {putbot $bot "valuechange $botnick floodmsg $chan $value"}
         }
         putserv "NOTICE $nick :'floodmsg' changed to '$value' for $chan."
         putflog $chan "-- RK: ($nick) changed 'floodmsg' for $chan to: $value."
      }
      repeatban {
         # ------------------------------------------------------------------
         # REPEATBAN variable (on/off)
         # ------------------------------------------------------------------
         if {$repeatban($chan) == 0} { 
            putserv "NOTICE $nick :Ban on single repeat now ENABLED. for channel $chan."
            putlog "-- RK: ($nick) toggled 'repeatban' ON for $chan."
            set repeatban($chan) 1
            foreach bot [bots] {
               if {$bot_scriptrunner($bot) == 1} {putbot $bot "valuechange $botnick repeatban $chan 1"}
            }
         } \
         else {
            putserv "NOTICE $nick :Ban on single repeat now DISABLED. for channel $chan."
            putflog $chan "-- RK: ($nick) toggled 'repeatban' OFF for $chan."
            set repeatban($chan) 0
            foreach bot [bots] {
               if {$bot_scriptrunner($bot) == 1} {putbot $bot "valuechange $botnick repeatban $chan 0"}
            }
         }
      }
      repeatbantime {
         # ------------------------------------------------------------------
         # REPEATBANTIME variable
         # ------------------------------------------------------------------  
         if { $value == "" } { 
            putserv "NOTICE $nick :Usage: /msg $botnick repeatset <#channel> repeatbantime <ban length>"
            return 
         }
         if { $value == $repeatbantime($chan) } { 
            putserv "NOTICE $nick :Setting for 'repeatbantime' on $chan is already $value."
            return 
         }
         if { $value < 0 || $value > 60 } { 
            putserv "NOTICE $nick :Value must be between 0 (permanent) and 60."
            return 
         }
         set repeatbantime($chan) $value
         foreach bot [bots] {
            if {$bot_scriptrunner($bot) == 1} {putbot $bot "valuechange $botnick repeatbantime $chan $value"}
         }
         putserv "NOTICE $nick :'repeatbantime' changed to '$value' for $chan."
         putflog $chan "-- RK: ($nick) changed 'repeatbantime' for $chan to: $value."
      }
      chartolerance {
         # ------------------------------------------------------------------
         # CHARACTER TOLERANCE variable
         # ------------------------------------------------------------------
         set value [string trim $value %]
         if { $value == "" } {
            putserv "NOTICE $nick :Usage: /msg $botnick repeatset <#channel> chartolerance <new value>"
            return 
         }
         if { $value == $chartolerance($chan) } { 
            putserv "NOTICE $nick :Setting for 'chartolerance' on $chan is already $value." 
            return 
         }
         if {$value <= 0 || $value > 100} { 
            putserv "NOTICE $nick :Value must be between 0 and 100."
            return 
         }
         set chartolerance($chan) $value
         foreach bot [bots] {
            if {$bot_scriptrunner($bot) == 1} {putbot $bot "valuechange $botnick chartolerance $chan $value"}
         }
         putserv "NOTICE $nick :'chartolerance' changed to '$value' for $chan."
         putflog $chan "-- RK: ($nick) changed 'chartolerance' for $chan to: $value."        
      }
      postolerance {
         # ------------------------------------------------------------------
         # POSITIONAL MATCHING TOLERANCE variable
         # ------------------------------------------------------------------
         set value [string trim $value %]
         if { $value == "" } {
            putserv "NOTICE $nick :Usage: /msg $botnick repeatset <#channel> postolerance <new value>"
            return 
         }
         if { $value == $postolerance($chan) } { 
            putserv "NOTICE $nick :Setting for 'postolerance' on $chan is already $value." 
            return 
         }
         if {$value <= 0 || $value > 100} { 
            putserv "NOTICE $nick :Value must be between 0 and 100."
            return 
         }
         set postolerance($chan) $value
         foreach bot [bots] {
            if {$bot_scriptrunner($bot) == 1} {putbot $bot "valuechange $botnick postolerance $chan $value"}
         }
         putserv "NOTICE $nick :'postolerance' changed to '$value' for $chan."
         putflog $chan "-- RK: ($nick) changed 'postolerance' for $chan to: $value."        
      }
      repeatonoff {
         # ------------------------------------------------------------------
         # REPEATONOFF variable (on/off)
         # ------------------------------------------------------------------
         if {$repeatonoff($chan) == 0} { 
            putserv "NOTICE $nick :Repeatkick'ing now ENABLED. for channel $chan."
            putflog $chan "-- RK: ($nick) toggled 'repeatkicking' ON for $chan."
            set repeatonoff($chan) 1
            foreach bot [bots] {
               if {$bot_scriptrunner($bot) == 1} {putbot $bot "valuechange $botnick repeatonoff $chan 1"}
            }
         } \
         else {
            putserv "NOTICE $nick :Repeatkick'ing now DISABLED. for channel $chan."
            putflog $chan "-- RK: ($nick) toggled 'repeatkicking' OFF for $chan."
            set repeatonoff($chan) 0
            foreach bot [bots] {
               if {$bot_scriptrunner($bot) == 1} {putbot $bot "valuechange $botnick repeatonoff $chan 0"}
            }
         }
      }
      floodonoff {
         # ------------------------------------------------------------------
         # FLOODONOFF variable (on/off)
         # ------------------------------------------------------------------
         if {$floodonoff($chan) == 0} { 
            putserv "NOTICE $nick :Floodkick'ing now ENABLED. for channel $chan."
            putflog $chan "-- RK: ($nick) toggled 'floodkicking' ON for $chan."
            set floodonoff($chan) 1
            foreach bot [bots] {
               if {$bot_scriptrunner($bot) == 1} {putbot $bot "valuechange $botnick floodonoff $chan 1"}
            }
         } \
         else {
            putserv "NOTICE $nick :Floodkick'ing now DISABLED. for channel $chan."
            putflog $chan "-- RK: ($nick) toggled 'floodkicking' OFF for $chan."
            set floodonoff($chan) 0
            foreach bot [bots] {
               if {$bot_scriptrunner($bot) == 1} {putbot $bot "valuechange $botnick floodonoff $chan 0"}
            }
         }
      }
      logging {
         # ------------------------------------------------------------------
         # LOGGING variable (on/off)
         # ------------------------------------------------------------------
         if {$logging($chan) == 0} { 
            putserv "NOTICE $nick :RK logging now ENABLED. for channel $chan."
            putflog $chan "-- RK: ($nick) toggled 'logging' ON for $chan."
            set logging($chan) 1
            foreach bot [bots] {
               if {$bot_scriptrunner($bot) == 1} {putbot $bot "valuechange $botnick logging $chan 1"}
            }
         } \
         else {
            putserv "NOTICE $nick :RK logging now DISABLED. for channel $chan."
            putflog $chan "-- RK: ($nick) toggled 'logging' OFF for $chan."
            set logging($chan) 0
            foreach bot [bots] {
               if {$bot_scriptrunner($bot) == 1} {putbot $bot "valuechange $botnick logging $chan 0"}
            }
         }
      }
      default {
         # This happens when a user tries to change a variable that doesn't exist.
         putserv "NOTICE $nick :Unknown variable/value combination."
         putserv "NOTICE $nick :See /msg $botnick repeatinfo <#channel>  ..for more info."
         return
      }
   }

   # Write changes to .ini file.
   create_ini "amend"
}

#---------------------------------------------------------------------------------------------
# BOTLINK: 'Check handling'
# This is used to determine whether or not it is safe for THIS bot to process events
# leading to the punishment of a user for repeating/flooding.  It is used to determine
# whether or not another bot is already handling it (using the 'bot_handling' variable)
#---------------------------------------------------------------------------------------------
proc check_handling {type} {
   global RKlinking bot_handling bot_scriptrunner

   # If bots are linking, and they jump into this procedure, if another is already in it, the other one(s)
   # will immediately stop all further processing.  This saves CPU cycles and effectively halves the work
   # done in normal processing between 2 shared bots (for example)

   if {$RKlinking == 1} {
      if {$bot_handling == 1} {
         set bot_handling 0
         return 1
      } else {
         foreach bot [bots] {
            if {$bot_scriptrunner($bot) == 1} {
               if {$type == 0} {putbot $bot "iamhandling 0"} else {putbot $bot "iamhandling 1"}
            }
         }
         return 0
      }
   } 
}
#---------------------------------------------------------------------------------------------
# BOTLINK: 'Clear handling'
# When processing is finished, ensure my own 'bot_handling' variable is cleared, and that
# all other bots are told to clear theirs too.
#---------------------------------------------------------------------------------------------
proc clear_handling {} {
   global RKlinking bot_handling bot_scriptrunner
   if {$RKlinking == 1} {
     # Clear RK handling locks
     set bot_handling 0
     foreach bot [bots] {
        if {$bot_scriptrunner($bot) == 1} {putbot $bot "closehandle"}
     }
   }
}

#---------------------------------------------------------------------------------------------
# PUBLIC repeat (normal channel text)
#---------------------------------------------------------------------------------------------
proc repeat_pubm {nick uhost hand chan text} {
  if [matchattr $hand f|f $chan] {return 0}
  global repeat_last repeat_num repeatkick repeatmsg repeatban repeatbantime repeatonoff inuse timercalled
  if {$repeatonoff([string tolower $chan]) == 0 } { return }
  set type ""
  set text [trimctrl $text]
  if [info exists repeat_last([set n [string tolower $nick]])] { 

      # Tell other bots I'm handling (or check to see if its being handled.
      set canprocess [check_handling 0]
      if {$canprocess == 1} { return 1 }
      
      set inuse 1
      if {[string compare [string tolower $repeat_last($n)] [string tolower $text]] == 0} {
            set type "EXACT MATCH"
            repeat_dostuff $nick $uhost $hand $chan $text $n $type
            set repeat_last($n) $text
            clear_handling
            return
         } \
         elseif {[mask_check $n $text [string tolower $repeat_last($n)]] == 1} {
            set type "Mask check FAIL"
            repeat_dostuff $nick $uhost $hand $chan $text $n $type
            set repeat_last($n) $text
            clear_handling
            return
         } \
         elseif {[letter_check $n $text $chan [string tolower $repeat_last($n)] 1 ""] == 1} {
            set type "Letter check FAIL"
            repeat_dostuff $nick $uhost $hand $chan $text $n $type
            set repeat_last($n) $text
            clear_handling
            return
         }
      if {$timercalled == 1} {
         catch {unset repeat_last}
         catch {unset repeat_num}
         set timercalled 0
      }
      set inuse 0

      # Clear "Im handling" botflags
      clear_handling

  }
  set repeat_num($n) 1
  set repeat_last($n) $text
}

#---------------------------------------------------------------------------------------------
# ACTION repeating (channel actions)
#---------------------------------------------------------------------------------------------
proc repeat_action {nick uhost hand dest keyword text} {
  global botnick altnick repeat_last repeat_num repeatkick repeatmsg repeatban repeatbantime repeatonoff inuse timercalled
  if {$repeatonoff([string tolower $dest]) == 0 } { return }
  set type ""
  set text [trimctrl $text]
  if {$dest == $botnick || $dest == $altnick || [matchattr $hand f|f $dest]} {return 0}
  if [info exists repeat_last([set n [string tolower $nick]])] {

      # Tell other bots I'm handling (or check to see if its being handled.
      set canprocess [check_handling 0]
      if {$canprocess == 1} { return 1 }

      set inuse 1     
      if {[string compare [string tolower $repeat_last($n)] [string tolower $text]] == 0} {
            set type "EXACT MATCH"
            repeat_dostuff $nick $uhost $hand $dest $text $n $type
            set repeat_last($n) $text
            clear_handling
            return
         } \
         elseif {[mask_check $n $text [string tolower $repeat_last($n)]] == 1} {
            set type "Mask check FAIL"
            repeat_dostuff $nick $uhost $hand $dest $text $n $type
            set repeat_last($n) $text
            clear_handling
            return
         } \
         elseif {[letter_check $n $text $dest [string tolower $repeat_last($n)] 1 ""] == 1} {
            set type "Letter check FAIL"
            repeat_dostuff $nick $uhost $hand $dest $text $n $type
            set repeat_last($n) $text
            clear_handling
            return
         } 
      if {$timercalled == 1} {
         catch {unset repeat_last}
         catch {unset repeat_num}
         set timercalled 0
      }
      set inuse 0
      
      # Clear "Im handling" botflags
      clear_handling
  }
  set repeat_num($n) 1
  set repeat_last($n) $text
}

#---------------------------------------------------------------------------------------------
# FLOODING
# NB. 'Flooding' is determined by the standard eggdrop 'flood-pub' setting for a given
#     channel.  There is no Repeatkick specific variables to set for setting the boundaries
#     of what constitutes a flood and what doesn't.
#---------------------------------------------------------------------------------------------
proc flood_check {nick uhost hand fludtype chan} {

   # Tell other bots I'm handling (or check to see if its being handled.
   set canprocess [check_handling 0]
   if {$canprocess == 1} { return 1 }

   global repeatkick repeat_num floodonoff repeat_last inuse timercalled
   # Channel flood.
   set inuse 1

   # *NEW* to version 2.15
   # Identification of nature of flood.
   switch $fludtype {
      ctcp { set type "CFlood" }
       msg { set type "MFlood" }
      join { set type "JFlood" }
   default { set type "Flood" } 
   }

   if [matchattr $hand f|f $chan] {return 1}
   if {$fludtype != "msg"} {
      if {$floodonoff([string tolower $chan]) == 0} { return 1 }
   }   

   set chan [string tolower $chan]
   set text ""
   set n [string tolower $nick]
   set repeat_num($n) $repeatkick($chan)
   set repeat_last($n) $text
   repeat_dostuff $nick $uhost $hand $chan $text $n $type

   if {$timercalled == 1} {
      catch {unset repeat_last}
      catch {unset repeat_num}
      set timercalled 0
   }
   set inuse 0

   # Clear "Im handling" botflags
   clear_handling

   # Note the next line is important to stop the normal flud action from taking place.
   return 1
}

#---------------------------------------------------------------------------------------------
# REPEAT/FLOOD punishment
# This is the main heart of Repeatkick, it is called when it has already been determined
# that someone on a channel the bot is monitoring, has repeated or flooded.  It also sets
# various userfile variables to help with the 'intelligent' banning (see check_banned)
#---------------------------------------------------------------------------------------------
proc repeat_dostuff {nick uhost hand chan text n type} {
   set chan [string tolower $chan]
   global repeat_last repeat_num repeatkick repeatmsg repeatban repeatbantime floodmsg bot_scriptrunner bot_handling botnick RKlinking
   set repeattempmsg $repeatmsg($chan)

   # Flood handling
   switch $type {
      CFlood { set repeattempmsg "CTCP Flooding" }
      JFlood { set repeattempmsg "Join Flooding" }
      MFlood { set repeattempmsg "Msg flooding" }
      Flood { set repeattempmsg $floodmsg($chan) }
   }

   if {[incr repeat_num($n)] >= $repeatkick($chan)} {   
        # Exit if another bot is handling this.
        set canprocess [check_handling 1]
        if {$canprocess == 1} { return 1 }

        # Does the user exist?.. if not create one.
        set banmask "*!*[string range $uhost [string first "@" $uhost] end]"
 
        # If ident matches, assume same user.
        # Stops several accounts being created by same person having different nicks
        set badhost [getchanhost $nick $chan]
        set joinedid [string range $badhost 0 [expr [string first "@" $badhost] - 1]]
        set trunc_nick ""
        foreach nortyuser [userlist] {
           if { [matchattr $nortyuser D] } {         
              set badident [getuser $nortyuser XTRA repeat_ident]      
              if { [string compare [string tolower $badident] [string tolower $joinedid]] == 0 \
               && [string compare [string tolower [getuser $nortyuser XTRA repeat_nick]] [string tolower [string range $nick 0 8]]] != 0} {
                 putflog $chan "-- RK: IDENT MATCH: Assuming '$nick' to be '[getuser $nortyuser XTRA repeat_nick]'.."
                 set trunc_nick [string range [getuser $nortyuser XTRA repeat_nick] 0 8]
                 # Add hostmask
                 setuser $trunc_nick HOSTS $banmask
              }
           }
        }
        
        if { $trunc_nick == "" } {
           # Ok cant find him by ident in the DB, search by nick now
           set find_him [finduser $banmask]
           set trunc_nick [string range $nick 0 8]

           if { $find_him == "*" } {
              adduser $trunc_nick $banmask
              putflog $chan "-- RK: Created new RK-userfile entry for '$nick'"
              setuser $trunc_nick XTRA repeat_nick $nick
              setuser $trunc_nick XTRA repeat_ident "[string range $uhost 0 [expr [string first "@" $uhost] - 1]]"
              setuser $trunc_nick XTRA repeat_oldhst $banmask
              # Set the "Delete me please!" flag
              chattr $trunc_nick +D
           }
        }        
   
        set repeatval [getuser $trunc_nick XTRA repeat_num]  
  
        # If auto-ban is set, ban immediately.
        if {$repeatban($chan)} {
          newchanban $chan $banmask RK $repeattempmsg $repeatbantime($chan)
          putserv "KICK $chan $nick :Banned: Repeating/Flooding.."
        } else {
           # Evaluate the type of kick.
           switch -exact $repeatval {
              >=3 {
                    newchanban $chan $banmask RK $repeattempmsg $repeatbantime($chan)
                    set temp [getuser $trunc_nick XTRA repeat_num]
                    incr temp
                    setuser $trunc_nick XTRA repeat_num $temp
                    putflog $chan "-- RK: Repeat from '$nick' ($chan) ($type).. Renewing 10 min. ban"
                    putserv "KICK $chan $nick :Banned: Repeating/Flooding.." 
                    flushmode $chan
                }
              2 {
                    newchanban $chan $banmask RK $repeattempmsg $repeatbantime($chan)
                    set temp [getuser $trunc_nick XTRA repeat_num]
                    incr temp
                    setuser $trunc_nick XTRA repeat_num $temp
                    putflog $chan "-- RK: 3rd repeat from '$nick' ($chan) ($type).. Kickbanning for 10 mins.."
                    putserv "KICK $chan $nick :Banned: Third strike.  He's outta there.."          
                    flushmode $chan
	        }        
              1 {
                    putserv "KICK $chan $nick :$repeattempmsg (FINAL warning)"
                    setuser $trunc_nick XTRA repeat_num 2
                    putflog $chan "-- RK: 2nd repeat/flood from '$nick' ($chan) ($type).. Kicking.."     
                }
        default {
                    putserv "KICK $chan $nick :$repeattempmsg (1st warning)"
                    setuser $trunc_nick XTRA repeat_num 1
                    putflog $chan "-- RK: Repeat/flood from '$nick' ($chan) ($type).. Kicking.." 
                }
           }
        }
        unset repeat_last($n)
        set repeat_num($n) 0
        #unset repeat_num($n)
    }
}

#---------------------------------------------------------------------------------------------
# Put FILE log.
# This procedure simply writes whatever is passed to it to the logfile for the channel
# (if 'logging' is on for the relevant channel)
#---------------------------------------------------------------------------------------------
proc putflog {chan text} {
   global logging logdir
   set chan [string tolower $chan]

   # Does directory exist?.. if not, create it.
   if {![file isdirectory $logdir]} {
      catch { file mkdir $logdir }
   }

   if {$logging($chan) == 1} {
      set chan [string trimleft $chan #]
      set fileopen [catch {set fd [open "$logdir/$chan.log" a+]}]
      if {$fileopen == 0} {
         puts $fd "([date]/[time]) $text"
         close $fd
      }
   }
   # Log will always been written out.
   putlog $text
}

#---------------------------------------------------------------------------------------------
# Nick change procedure - this is used to ensure syncronisation in the repeat_num and
# repeat_last variable arrays. (ie. that they are carried over from 'oldnick' to 'newnick'
#---------------------------------------------------------------------------------------------
proc repeat_nick {nick uhost hand chan newnick} {
  if [matchattr $hand f|f $chan] {return 0}
  global repeat_last repeat_num
  catch {set repeat_last([set nn [string tolower $newnick]]) \
         $repeat_last([set on [string tolower $nick]])}
  catch {unset repeat_last($on)}
  catch {set repeat_num($nn) $repeat_num($on)}
  catch {unset repeat_num($on)}
  check_banned $newnick $uhost $hand $chan
}

#---------------------------------------------------------------------------------------------
# Timer firing procedure - this is called when a channel timer is fired.  It simply 'unsets'
# the 'repeat_num' and 'repeat_last'.  NB.  It is probably a good idea, until a later revision
# at least, to have the same 'repeattime' values for each channel, otherwise you may find the
# bot missing repeats (because repeat_num and repeat_last are currently GLOBAL, ie across
# all channels).  This will be fixed later.
#---------------------------------------------------------------------------------------------
proc repeat_timr {} {
  global repeat_last repeattime repeattimer inuse timercalled

  # Arrays are only reset if they are 'unlocked'
  if {$inuse == 0} {
     catch {unset repeat_last}
     catch {unset repeat_num}
  } else {
     set timercalled 1
  }

  foreach chan [channels] {
     if {[string match "*$repeattimer($chan)*" [utimers]] == 0} {
        set repeattimer($chan) [utimer $repeattime($chan) repeat_timr]
        return 
     }
  }
}
#---------------------------------------------------------------------------------------------
# Repeatkick clean procedure - this is called every 6 hours or so (by default), and what
# it does is remove everyone from the userfile that has flag +D (assuming they are not an 
# op or anything).  This means that all records of repeating users are cleared, so they 
# don't get auto-rebanned on a further repeat (if they've been banned earlier in the day, etc)
#---------------------------------------------------------------------------------------------
proc repeatk_clean {} {
  # The purpose of this function is to reset the 'repeatnum' values
  # every day (otherwise everyone would get shitlisted eventually).
  # This is done by deleting all userrecords not marked as FRIEND or
  # OP (thereby saving space).
  
  putlog "Beginning repeatkick reinitialisation.."
  set total_before [countusers]

  set deleted_users 0
  foreach user [userlist] {
     if { [matchattr $user D] && ![matchattr $user o]} {
        deluser $user
        incr deleted_users
     }
  }
  putlog "-- RK: Initialisation complete.  Removed $deleted_users users from the orig. db of $total_before."
  if ![regexp repeatk_clean [timers]] {
     timer 360 repeatk_clean 
  }
}

#---------------------------------------------------------------------------------------------
# CHECKING ALGORITHMS
#
# - Mask checking
# This takes as input the text of whatever a user has just said and compares it to whatever
# was previously said with the mask "*<last said>*", so essentially if 'new text' appears in 
# 'old text', with anything at either side of it (left/right), then that fails MASK checking.
# example: oldtext 'hello', newtext 'mmmhello hmm'
#
# - Letter checking
# This is more advanced.  Basically it uses two pre-defined channel variables, chartolerance
# and postolerance to determine whether or not 'text' fails LETTER checking.  It works by
# comparing each individual character in 'newtext' with whats stored in 'oldtext' (character
# tolerance), and also noting the position of each character in relation to whats stored in
# 'oldtext' (positional tolerance).  If both of these values are high (ie. most/all of the 
# same letters in the same places), then 'newtext' FAILS letter checking.
#---------------------------------------------------------------------------------------------

# Mask checking
proc mask_check {n text repeattext} {
  global repeat_last repeat_num repeatkick repeatmsg repeatban repeatbantime
  if {(([string match [string tolower "*$repeattext*"] [string tolower $text]] == 1) && ([string length $repeattext] >= 4))} {
     return 1
  }
  return 0
} 

# Letter checking
proc letter_check {n text chan repeattext type oldhost} {
  global repeat_last repeat_num repeatkick repeatmsg repeatban repeatbantime chartolerance postolerance debug_mode
  set matching 0
  set alphabet "abcdefghijklmnopqrstuvwxyz !@#$%^&*()_+|{}:<>,./\0123456789"
  set chan [string tolower $chan]

  # Initialise alpabet values
  for {set ctr 0} {$ctr <= [string length $alphabet]} {incr ctr} {
     set alpha($ctr) -1
     set rep_alpha($ctr) -1
     set alpha_pos($ctr) -1
     set rep_alpha_pos($ctr) -1
  }

  # Get alphabet values for 'text'
  for {set ctr 0} {$ctr <= [string length $text]} {incr ctr} {
     set foundchar [string tolower [string index $text $ctr]]
     set foundpos [string first $foundchar $alphabet]
     if {$foundpos != -1} {incr alpha($foundpos)}
     if {($foundpos != -1) && ($alpha_pos($foundpos) == -1)} {set alpha_pos($foundpos) $ctr}
  }
  
  # Do the same for 'repeat_last'
  for {set ctr 0} {$ctr <= [string length $repeattext]} {incr ctr} {
     set foundchar [string tolower [string index $repeattext $ctr]]
     set foundpos [string first $foundchar $alphabet]
     if {$foundpos != -1} {incr rep_alpha($foundpos)}
     if {($foundpos != -1) && ($rep_alpha_pos($foundpos) == -1)} {set rep_alpha_pos($foundpos) $ctr}
  }

  # Make comparisons.
  set letter_sim 0
  set pos_sim 0
  for {set ctr 0} {$ctr <= [string length $alphabet]} {incr ctr} {
    if {($alpha($ctr) == $rep_alpha($ctr)) && ($alpha($ctr) != -1 && $rep_alpha($ctr) != -1)} { incr letter_sim }
    if {($alpha_pos($ctr) == $rep_alpha_pos($ctr)) && ($alpha_pos($ctr) != -1 && $rep_alpha_pos($ctr) != -1)} { incr pos_sim }
  }  

  set max_letsim 0 
  set max_possim [string length $repeattext]
  # Get maximum possible.
  for {set ctr 0} {$ctr <= [string length $alphabet]} {incr ctr} {
    if {($alpha($ctr)) != -1} { incr max_letsim }
  }

  # Percentages
  if {$max_letsim > 0} {set sim_perc [expr ($letter_sim * 100) / $max_letsim]} else {set sim_perc 0}
  if {$max_possim > 0} {set pos_perc [expr ($pos_sim * 100) / $max_possim]} else {set pos_perc 0}

  unset alpha
  unset rep_alpha
  unset alpha_pos
  unset rep_alpha_pos
  
  # Trim excess from one to another.
  set sim_excess [expr $sim_perc - $chartolerance([string tolower $chan])]
  set pos_excess [expr $pos_perc - $postolerance([string tolower $chan])]
  set old_sim_perc 0
  set old_pos_perc 0
  if {($sim_excess > 0) && ($pos_excess > 0)} { # Do nothing } \
     elseif {$sim_excess > 0} {
        set old_pos_perc $pos_perc
        set pos_perc [expr $pos_perc + $sim_excess]
        set old_sim_perc $sim_perc
        set sim_perc $chartolerance([string tolower $chan])
     } \
     elseif {$pos_excess > 0} {
        set old_sim_perc $sim_perc
        set sim_perc [expr $sim_perc + $pos_excess]
        set old_pos_perc $pos_perc
        set pos_perc $postolerance([string tolower $chan])
     }

  if {$sim_perc > 100} {set sim_perc 100} \
     elseif {$pos_perc > 100} {set pos_perc 100}

  # Included for DEBUG mode.
  if {$debug_mode == 1} {
     putflog $chan "-- RK: $n: char: $letter_sim/$max_letsim ($sim_perc%), pos: $pos_sim/$max_possim ($pos_perc%), repeat_num: $repeat_num($n).. (cur: '$text', last: '$repeattext')"
  }

  if {($sim_perc >= $chartolerance([string tolower $chan])) && ($pos_perc >= $postolerance([string tolower $chan]))} {
     if {($type == 0 && [isban $oldhost $chan]) || ($repeat_num($n) == [expr $repeatkick($chan) - 1])} {
        putflog $chan "-- RK: TOLERANCE FAIL ($n) - char: $letter_sim/$max_letsim ($old_sim_perc%/$chartolerance([string tolower $chan])%), pos: $pos_sim/$max_possim ($old_pos_perc%/$postolerance([string tolower $chan])%)"
        if {$sim_excess > 0} {
           putflog $chan "-- RK: Overflow of $sim_excess% on char. tolerance (was $old_sim_perc%) - moved to pos tolerance. (was $old_pos_perc%, now $pos_perc%)"
           unset old_sim_perc
           unset old_pos_perc
        } \
        elseif {$pos_excess > 0} {
           putflog $chan "-- RK: Overflow of $pos_excess% on pos. tolerance (was $old_pos_perc%) - moved to char tolerance. (was $old_sim_perc%, now $sim_perc%)"
           unset old_sim_perc
           unset old_pos_perc
        }
     }
     # Cleanup: unset old variables.
     unset pos_perc
     unset sim_perc
     unset letter_sim
     unset max_letsim
     unset pos_sim
     unset max_possim     
     return 1
  } else {
     # Cleanup: unset old variables.
     unset pos_perc
     unset sim_perc
     unset letter_sim
     unset max_letsim
     unset pos_sim
     unset max_possim    
     return 0
  }
} 

#---------------------------------------------------------------------------------------------
# TRIM text procedure - this is used to trim the incoming 'text' of all unnecessary gunff
# like control characters, mIRC colour formatting etc.  It also removes all white space
# between words, and converts it all to lowercase.
#---------------------------------------------------------------------------------------------
# Used for all mask/letter checking procs.

proc trimctrl {text} { 
  # 1) Remove any control characters (color chars) from the text..
  set text [string trim $text "0-15"]

  # 2) Now remove all whitespace..
  # Remove trailing/preceding whitespace, etc
  set text [string trim $text]

  # 3) Now remove all whitespace between letters
  for {set x 0} {$x<=[string length $text]} {incr x} {
     if { "[string index $text $x]" != " " } { append finaltext [string index $text $x] }
  }     
  
  # 4) Return contents (lowercased) 
  return $finaltext
}


#---------------------------------------------------------------------------------------------
# INITIAL BRINGUP procedure - this is called when the bot first starts, or on a restart 
# (not on a .rehash).  It initialises several variables, sets timers and calls for other
# bots on the BOTNET to identify themselves if they are running Repeatkick also..
#---------------------------------------------------------------------------------------------
proc firstinit {} {
  # Just calls the above function.
  # Used when the bot is brought online from a restart etc.

  global initstart
  repeatk_clean
}
putlog "-------------------------------------------------------"
putlog "Repeat/Floodkick Protection $version_num INITIALISING.."
putlog "-------------------------------------------------------"

# Kill all existing timers
set ctr 0
foreach timr [utimers] {
   if {[lindex $timr 1] == "repeat_timr"} {
      killutimer [lindex $timr 2]
      incr ctr
   }
}
if {$ctr > 0} {putlog "-- RK: Killed $ctr old 'repeat_timr' utimers successfully."}
unset ctr

# Parse .ini file
parse_ini

foreach chan [channels] {
   if {$repeattimer($chan) == 0} {     # thanks to jaym
      set repeattimer($chan) [utimer $repeattime($chan) repeat_timr]
   }
}

if ![regexp repeatk_clean [timers]] {
  set initstart [utimer 1 firstinit]
  timer 360 repeatk_clean 
}

foreach bot [bots] {
   # Pre-initialise 'bot running RK' array
   set bot_scriptrunner($bot) 0
}

set inittime [unixtime]

# Call for other script runners to identify
if {$RKlinking == 1} {
   bot_check_usage
} else {
   # Unbind all bot commands
   unbind bot - iamhandling bot_being_handled
   unbind bot - closehandle bot_close_handle
   unbind bot - filesync bot_filesync
   unbind bot - whoishere bot_respond_usage
   unbind bot - iamhere bot_using_script
   unbind bot - wantfile bot_wantsfile
   unbind bot - ackreload bot_ackreload
   unbind bot - valuechange bot_valuechange
   unbind bot - readthis bot_readthis
   unbind bot - inisettings bot_inireceive
   unbind bot - inifinished bot_inifinished
   unbind rcvd b * bot_file_received
   unbind link - * bot_single_check_usage
   putlog "-- RK:  RK Botlinking disabled."
}

#---------------------------------------------------------------------------------------------
# Script ends here. :)
# (C) Darren Coleman 1999
#
# http://www.durzel.dircon.co.uk
# Patches, etc will be released on the above website
#---------------------------------------------------------------------------------------------
